//
// THIS FILE IS COPYRIGHT 1994, 1995 BY TELENOR RESEARCH, NORWAY
//
//                        ALL RIGHTS RESERVED.
//
// For rights granted to the public see the COPYRIGHT file
//
//
// LexAnalyzer.cc	The lexical analyzer for the FFW search engine.
//
// Author: Haiyan Yang

#include <iostream.h>
#include <lang.h>
#include "LexAnalyzer.h"
#include <convert.h>

extern int strcmp8(const unsigned char *s1, const unsigned char *s2);

Lex::Lex(String& is)
{
    input = is;
    input_ptr = 0;
    curr = NULL;
}

Token* Lex::getToken()
{
    int quoted=0, numChars=0, numTokens=0, isInWord=0;
    unsigned char word[160];
    unsigned char c=0;

    while (input_ptr < input.length()) {
        c = ToLower[(unsigned char)input[input_ptr++]];
	
	if (c == '\0') {
	    isInWord = 0;
	} else if (quoted) {
	    switch (c) {
	    case '"':
		isInWord = 0;
		break;
	    default:
		word[numChars++] = c;
		isInWord = 1;
		break;
	    }
	} else {
	    switch (c) {
	    case '(':
		isInWord = 0;
		if (numChars > 0) {
		    input_ptr--;
		} else {
		    return new LeftParenToken();
		}
		break;
	    case ')':
		isInWord = 0;
		if (numChars > 0) {
		    input_ptr--;
		} else {
		    return new RightParenToken();
		}
		break;
	    case '"':
		quoted = 1;
		isInWord = 0;
		break;
	    case ' ':	// Whitespace
	    case '\t':	// ...
	    case '\n':	// ...
		isInWord = 0;
		break;
	    default:
		word[numChars++] = c;
		isInWord = 1;
		break;
	    }
	}

	if (input_ptr == input.length())
	    isInWord = 0;
	     
	if (!isInWord && numChars > 0) { // If we were in a word
	    word[numChars] = '\0';
	    
	    if (!quoted) {
		if (strcmp8(word, And) == 0) {
		    return new AndToken();
		} else if (strcmp8(word, Or) == 0) {
		    return new OrToken();
		} else if (strcmp8(word, Not) == 0) {
		    return new NotToken();
		} else {
		    return new TextToken((char *)word);
		}
	    } else {
		return new TextToken((char *)word);
	    }
	}
    }

    return NULL;	// EOF reached, numChars gotta be 0 otherwise
			// the 'if we were in a word' test would have
			// evaluated to true and returned a token.
}


Token* Lex::currToken()
{
    static int first = 1;

    if (first) {
	curr = getToken();
	first = 0;
    }

    return curr;
}

Token* Lex::advanceToken()
{
    return curr = getToken();
}

