//
// THIS FILE IS COPYRIGHT 1994, 1995 BY TELENOR RESEARCH, NORWAY
//
//                        ALL RIGHTS RESERVED.
//
// For rights granted to the public see the COPYRIGHT file
//
//
// Parser.cc	The parser for the FFW search engine
//		Builds and returns the syntax tree
//
// Author: Haiyan Yang

#include <iostream.h>
#include <stdlib.h>
#include <lang.h>
#include "Parser.h"
#include "LexAnalyzer.h"

#define TOKEN	(lex->currToken() ? lex->currToken()->type() : EOS)
#define NEXT	lex->advanceToken()

// The grammar implemented is:
//
//   orterm = andterm | andterm OR orterm
//  andterm = notterm | notterm AND andterm
//  notterm = primary | NOT primary
//  primary = text | ( orterm )
//
// where 'orterm' is dedicated to be the starting non-terminal.

//
// Parser functions
//
TermNode* QueryParser::orterm()
{
    TermNode* n = andterm();

    if (TOKEN == OR) {
	NEXT;
	return new OrTermNode(n, orterm());
    }

    return n;
}

TermNode* QueryParser::andterm()
{
    TermNode* n = notterm();

    if (TOKEN == AND) {
	NEXT;
	return new AndTermNode(n, andterm());
    }

    return n;
}

TermNode* QueryParser::notterm()
{
    if (TOKEN == NOT) {
	NEXT;
	return new NotTermNode(primary());
    }

    return primary();
}

TermNode* QueryParser::primary()
{
    if (TOKEN == TEXT) {
	TermNode* n = new TextTermNode(((TextToken*)lex->currToken())
				       ->getWord());
	NEXT;
	return n;
    } else if (TOKEN == LEFTPAREN) {
	NEXT;
	TermNode* n = orterm();
	if (TOKEN != RIGHTPAREN) {
	    cout.form(MissingRightParantes, lex->getInputPtr(),
			 (char *)lex->getInput());
	    exit(1);
	}
	NEXT;
	return n;
    } else if (TOKEN == EOS) {
	cout.form(ParseError, lex->getInputPtr(),
                     (char *)lex->getInput());
	exit(1);
    } else {
	cout.form(UnexpectedSymbol, lex->getInputPtr(),
		     (char *)lex->getInput());
	exit(1);
    }
}


//
// Build and return syntax tree
//
SetNode* QueryParser::parse(String& is)
{
    TermNode*	termTree;
    SetNode*	setTree;

    // First select input stream...
    lex = new Lex(is);

    // Build parse tree and then syntax tree representing
    // the logical structure of the search expression.
    termTree = orterm();
    if (TOKEN != EOS) {		// We didn't parse everything?
	cout.form(ParseError, lex->getInputPtr(),
		     (char *)lex->getInput());
	exit(1);
    }

    // Transform syntax tree into a set expression tree
    setTree = termTree->build();

    // Check errors on setTree
    if (setTree->isNot()) {
	cout.form(NotNotReduced);
	exit(1);
    }

    // Return syntax tree to the interpreter
    return setTree;
}
