//
// THIS FILE IS COPYRIGHT 1994, 1995 BY TELENOR RESEARCH, NORWAY
//
//                        ALL RIGHTS RESERVED.
//
// For rights granted to the public see the COPYRIGHT file
//
//
// Token.h	The token class for the FFW lexical analyzer
//
// Author: Haiyan Yang

#ifndef _TOKEN_H
#define _TOKEN_H

#include <stdio.h>
#include <ctype.h>
#include <iostream.h>
#include <String.h>

typedef enum { EOS=0, AND, OR, NOT, TEXT, LEFTPAREN, RIGHTPAREN } token_t;


class Token
{
public:
    virtual String name() const = 0;
    virtual token_t type() const = 0;
};


class TextToken : public Token
{
private:
    String _str;
    String _name;

public:
// Constructors and destructors
    TextToken(String str);
    // Access methods
    String getWord() const { return _str; }
    unsigned char *getWord() { return (unsigned char *)(char *)_str; }
    String name() const { return _name; }
    virtual ~TextToken() { }
    token_t type() const { return TEXT; }
};


class AndToken : public Token
{
public:
    String name() const { return String("And()"); }
    token_t type() const { return AND; }
};

class OrToken : public Token
{
public:
    String name() const { return String("Or()"); }
    token_t type() const { return OR; }
};

class NotToken : public Token
{
public:
    String name() const { return String("Not()"); }
    token_t type() const { return NOT; }
};

class LeftParenToken : public Token
{
public:
    String name() const { return String("LeftParen()"); }
    token_t type() const { return LEFTPAREN; }
};

class RightParenToken : public Token
{
public:
    String name() const { return String("RightParen()"); }
    token_t type() const { return RIGHTPAREN; }
};

inline TextToken::TextToken(String str)
{
    _str = str;
    _name = "Text(" + str + ")";
}

#endif /* TOKEN_H */
